#!/bin/bash
# vars.sh                                                               2012-01-26 AgF
# (c) Copyright 2012 by Agner Fog. GNU General Public License www.gnu.org/licenses

# Code to initialize environment variables etc. inside test scripts
# NOTE: run as . inittest.sh, not as ./inittest.sh

# Compile A file if modified
if [ PMCTestA.cpp -nt a64.o ] ; then
g++ -O2 -c -m64 -oa64.o PMCTestA.cpp
fi

if [ PMCTestA.cpp -nt a32.o ] ; then
g++ -O2 -c -m32 -oa32.o PMCTestA.cpp
fi

# create results directory if it doen's exist
if [ ! -d results ] ; then mkdir results ; fi

# Variables that depend on CPU brand
# CPUbrand: AMD, Intel, VIA
# blockports: execution ports or pipes that we want to block in order to detect which port or pipe a specific instruction uses
# PMCs: list of PMC counters to use during test. See counter definitions in PMCTestA.cpp (separated by comma without space)
# PMClist: multiple sets of PMC counters if we can't have all desired counters simultaneously (lists separated by space)
if   [ `grep -c -i "AuthenticAMD" /proc/cpuinfo ` -ne 0 ] ; then
  CPUbrand=AMD 
  blockports="0 1"
  PMCs="9,100,162,150"
  PMClist="9,100,160,150  9,100,161,151 9,100,162,152  9,100,162,153"

elif [ `grep -c -i "GenuineIntel" /proc/cpuinfo ` -ne 0 ] ; then
  CPUbrand=Intel
  blockports="0 1 5"
  PMCs="1,9,100,150,151"
  PMClist="1,9,100,150,151  1,9,150,151,155"

elif [ `grep -c -i "CentaurHauls" /proc/cpuinfo ` -ne 0 ] ; then
  CPUbrand=VIA
  blockports=
  PMCs="9,100"
  PMClist="9,100"
else
  CPUbrand=unknown
  blockports=
  PMCs="9,100"
  PMClist="9,100"
fi

